#!/usr/bin/env python
import sys
from pyltfs2 import LTFS

if len(sys.argv) < 3:
    print 'Syntax: addtape2db.py <xml file> <name> <customa> ... <customd>'
    sys.exit(1)

sys.argv += ['', '', '', '', '', '', None]

args = sys.argv[1:7]

print """Adding tape
===========
XML      : %s
Name     : %s
Custom A : %s
Custom B : %s
Custom C : %s
Custom D : %s
===========
""" % tuple(args)
#Checksums: %s

url = 'mysql+mysqlconnector://root:123@localhost/syslink_tapelibrary'
ltfs = LTFS(url)

try:
    args[0] = open(args[0]).read()
except:
    print 'Can\'t read XML file'
    sys.exit(1)

#CHECKSUM_FILE = '/var/lib/syslink/archive/checksums'
ltfs.delete_tape(name=args[1])
ltfs.create_tape(
    *args
)

print 'Done!'



# -*- coding: utf-8 -*-
"""
    pyte
    ~~~~

    `pyte` implements a mix of VT100, VT220 and VT520 specification,
    and aims to support most of the `TERM=linux` functionality.

    Two classes: :class:`~pyte.streams.Stream`, which parses the
    command stream and dispatches events for commands, and
    :class:`~pyte.screens.Screen` which, when used with a stream
    maintains a buffer of strings representing the screen of a
    terminal.

    .. warning:: From ``xterm/main.c`` «If you think you know what all
                 of this code is doing, you are probably very mistaken.
                 There be serious and nasty dragons here» -- nothing
                 has changed.

    :copyright: (c) 2011 by Selectel, see AUTHORS for more details.
    :license: LGPL, see LICENSE for more details.
"""

__all__ = ("Screen", "DiffScreen", "HistoryScreen",
           "Stream", "ByteStream", "DebugStream",
           "ctrl", "esc", "mo", "g", "cs")

from . import (
    control as ctrl,
    charsets as cs,
    escape as esc,
    graphics as g,
    modes as mo
)
from .screens import Screen, DiffScreen, HistoryScreen
from .streams import Stream, ByteStream, DebugStream


if __debug__:
    def dis(chars):
        """A :func:`dis.dis` for terminals.

        >>> dis(u"\u0007")
        BELL
        >>> dis(u"\x9b20m")
        SELECT-GRAPHIC-RENDITION 20
        """
        if isinstance(chars, unicode):
            chars = chars.encode("utf-8")

        return DebugStream().feed(chars)



from ajenti.api import plugin
from ajenti.plugins.dashboard.api import ConfigurableWidget

from api import RAIDManager


@plugin
class LSIWidget (ConfigurableWidget):
    name = 'LSI MegaRAID'
    icon = 'hdd'

    def on_prepare(self):
        self.mgr = RAIDManager.get()
        self.append(self.ui.inflate('megaraid:widget'))

    def on_start(self):
        self.mgr.refresh()
        self.find('variant').text = self.config['variant']
        arr = self.mgr.find_array(self.config['variant'])
        if arr:
            self.find('value').text = arr.state

    def create_config(self):
        return {'variant': ''}

    def on_config_start(self):
        self.mgr.refresh()
        lst = list(self.mgr.list_arrays())
        v_list = self.dialog.find('variant')
        v_list.labels = lst
        v_list.values = lst
        v_list.value = self.config['variant']

    def on_config_save(self):
        self.config['variant'] = self.dialog.find('variant').value



import os
import threading
from SocketServer import UnixStreamServer
from BaseHTTPServer import BaseHTTPRequestHandler

from ajenti.api import *
from ajenti.plugins import manager
from ajenti.util import public


@public
@interface
class IPCHandler (object):
    """
    Interface for custom IPC endpoints
    """

    def get_name(self):
        """
        Should return short identifier of IPC endpoint:

        $ ajenti-ipc <endpoint-name> <args>
        """

    def handle(self, args):
        """
        Override to handle IPC requests

        :param args: list of `str` parameters
        """


class Handler (BaseHTTPRequestHandler):
    def do_GET(self):
        name, args = self.path.split('/')
        args = args.decode('base64').splitlines()
        for h in IPCHandler.get_all(manager.context):
            if h.get_name() == name:
                try:
                    result = h.handle(args)
                    self.send_response(200, 'OK')
                    self.end_headers()
                    self.wfile.write(result)
                except Exception, e:
                    self.send_response(500, 'Error')
                    self.end_headers()
                    self.wfile.write(str(e))
                break
        else:
            self.send_response(404, 'Handler not found')
            self.end_headers()
        self.wfile.close()

    def log_request(self, *args):
        pass


@public
@plugin
class IPCServer (BasePlugin):
    def start(self):
        t = threading.Thread(target=self.run)
        t.daemon = True
        t.start()

    def run(self):
        socket_path = '/var/run/ajenti-ipc.sock'
        if os.path.exists(socket_path):
            os.unlink(socket_path)

        server = UnixStreamServer(socket_path, Handler)
        os.chmod(socket_path, 0700)
        server.serve_forever()



#coding: utf-8
import subprocess
import re

from ajenti.api import BasePlugin, plugin
from ajenti.api.sensors import Sensor
from ajenti.plugins.dashboard.api import ConfigurableWidget
from ajenti.util import *


class LMSensors (BasePlugin):
    re_temp = re.compile(r'^(?P<name>.+?):\s+\+(?P<value>[\d.]+).+$')

    @cache_value(3)
    def get(self):
        try:
            lines = subprocess.check_output(['sensors']).splitlines()
        except subprocess.CalledProcessError, e:
            return {}  # sensors not configured
        r = {}
        for l in lines:
            m = self.re_temp.match(l)
            if m:
                r[m.groupdict()['name']] = float(m.groupdict()['value'])
        return r


@plugin
class Sensors (Sensor):
    id = 'lm-sensors'
    timeout = 3

    def init(self):
        self.lm = LMSensors()

    def measure(self, variant=None):
        try:
            return self.lm.get()[variant]
        except KeyError:
            return 0

    def get_variants(self):
        return self.lm.get().keys()


@plugin
class TempWidget (ConfigurableWidget):
    name = 'Temperature'
    icon = 'fire'

    def on_prepare(self):
        self.sensor = Sensor.find('lm-sensors')
        self.append(self.ui.inflate('lm_sensors:widget'))

    def on_start(self):
        value = self.sensor.value(self.config['sensor'])
        self.find('name').text = self.config['sensor']
        self.find('value').text = '+%.1f °C' % value

    def create_config(self):
        return {'sensor': ''}

    def on_config_start(self):
        service_list = self.dialog.find('sensor')
        service_list.labels = service_list.values = self.sensor.get_variants()
        service_list.value = self.config['sensor']

    def on_config_save(self):
        self.config['sensor'] = self.dialog.find('sensor').value



import logging
import os
import platform as pyplatform
import random
import signal
import subprocess

__version__ = '1.2.8'

# Global state

config = None
""" Loaded config, is a `reconfigure.items.ajenti.AjentiData` """

version = None
""" Ajenti version """

platform = None
""" Current platform """

platform_unmapped = None
""" Current platform without "Ubuntu is Debian"-like mapping """

platform_string = None
""" Human-friendly platform name """

installation_uid = None
""" Unique installation ID """

server = None
""" Web server """

debug = False
""" Debug mode """

edition = 'vanilla'


__all__ = ['config', 'platform', 'platform_string', 'platform_unmapped', 'installation_uid', 'version', 'server', 'debug', 'init', 'exit', 'restart']


def detect_version():
    p = subprocess.Popen('git describe --tags 2> /dev/null',
            shell=True,
            stdout=subprocess.PIPE)
    if p.wait() != 0:
        return __version__
    return p.stdout.read().strip('\n ')


def detect_platform():
    base_mapping = {
        'gentoo base system': 'gentoo',
        'centos linux': 'centos',
        'mandriva linux': 'mandriva',
        'elementary os': 'ubuntu',
        'trisquel': 'ubuntu',
        'linaro': 'ubuntu',
        'linuxmint': 'ubuntu',
        'redhat enterprise linux': 'rhel',
        'red hat enterprise linux server': 'rhel',
        'fedora': 'rhel',
        'olpc': 'rhel',
        'xo-system': 'rhel',
    }

    platform_mapping = {
        'ubuntu': 'debian',
        'rhel': 'centos',
    }

    if pyplatform.system() != 'Linux':
        res = pyplatform.system().lower()
        return res, res

    dist = ''
    (maj, min, patch) = pyplatform.python_version_tuple()
    if (maj * 10 + min) >= 26:
        dist = pyplatform.linux_distribution()[0]
    else:
        dist = pyplatform.dist()[0]

    if dist == '':
        try:
            dist = subprocess.check_output(['strings', '-4', '/etc/issue']).split()[0]
        except:
            dist = 'unknown'

    res = dist.strip(' \'"\t\n\r').lower()
    if res in base_mapping:
        res = base_mapping[res]

    res_mapped = res
    if res in platform_mapping:
        res_mapped = platform_mapping[res]
    return res, res_mapped


def detect_platform_string():
    try:
        return subprocess.check_output(['lsb_release',  '-sd'])
    except:
        return subprocess.check_output(['uname', '-mrs'])


def check_uid():
    file = '/var/lib/ajenti/installation-uid'
    if not os.path.exists(file):
        uid = str(random.randint(1, 9000 * 9000))
        try:
            open(file, 'w').write(uid)
        except:
            uid = '0'
    else:
        uid = open(file).read()
    return uid


def init():
    import ajenti
    ajenti.version = detect_version()
    if ajenti.platform is None:
        ajenti.platform_unmapped, ajenti.platform = detect_platform()
    else:
        logging.warn('Platform ID was enforced by commandline!')
        ajenti.platform_unmapped = ajenti.platform
    ajenti.platform_string = detect_platform_string()
    ajenti.installation_uid = check_uid()


def exit():
    os.kill(os.getpid(), signal.SIGQUIT)


def restart():
    server.restart_marker = True
    server.stop()



import copy

from ajenti.api import *
from ajenti.ui.element import p, UIElement
from ajenti.util import *


def is_bound_context(el):
    return '{binder}context' in el.properties and el.properties['{binder}context']


def is_bound(el):
    if el.typeid.startswith('bind:'):
        return True
    for prop in el.properties.keys():
        if prop == 'bind' or prop.startswith('{bind}') or is_bound_context(el):
            if el.properties[prop]:
                return True
    return False


@public
class Binding (object):
    """
    A base class for bindings. Binding is a link between a Python object attribute and Ajenti UI element's property.

    :param object: a Python object
    :param attribute: attribute name
    :param ui: Ajenti :class:`ajenti.ui.UIElement`
    """

    def __init__(self, object, attribute, ui):
        self.object = object
        self.attribute = attribute
        self.ui = ui

    def get(self):
        """
        :returns: value of the bound attribute
        """
        return getattr(self.object, self.attribute)

    def set(self, value):
        """
        Sets value of the bound attribute
        """
        try:
            setattr(self.object, self.attribute, value)
        except Exception:
            raise Exception('Binder set failed: %s.%s = %s' % (self.object, self.attribute, repr(value)))

    def populate(self):
        """
        Should update the UI with attribute's value
        """

    def unpopulate(self):
        """
        Should revert UI to normal state
        """

    def update(self):
        """
        Should update the attribute with data from the UI
        """


@public
class PropertyBinding (Binding):
    """
    A simple binding between UI element's property and Python object's attribute

    :param property: UI property name. If ``None``, property is deduced from ``bindtypes``
    """

    def __init__(self, obj, attribute, ui, property=None):
        Binding.__init__(self, obj, attribute, ui)
        if property is None:
            # find a property with matching bindtypes
            v = self.__get_transformed()
            for prop in ui.property_definitions.values():
                if prop.bindtypes:
                    # nb: we can't guess the type for None
                    if type(v) in prop.bindtypes or (v is None) or (object in prop.bindtypes):
                        self.property = prop.name
                        break
            else:
                raise Exception('Cannot bind %s.%s (%s, = %s) to %s' % (repr(obj), attribute, repr(type(v)), repr(v), ui))
        else:
            self.property = property
        self.oneway = ui.bindtransform is not None

    def __get_transformed(self):
        return self.ui.bindtransform(self.get()) if self.ui.bindtransform else self.get()

    def populate(self):
        self.old_value = self.get()
        setattr(self.ui, self.property, self.__get_transformed())

    def update(self):
        if self.oneway:
            return
        new_value = getattr(self.ui, self.property)
        # avoid unnecessary sets
        if new_value != self.old_value:
            self.set(new_value)


class DictValueBinding (PropertyBinding):
    def get(self):
        return self.object.get(self.attribute, None)

    def set(self, value):
        self.object[self.attribute] = value


@public
class ListAutoBinding (Binding):
    """
    Binds values of a collection to UI element's children consecutively, using :class:`Binder`
    """

    def __init__(self, object, attribute, ui):
        Binding.__init__(self, object, attribute, ui)
        self.binders = {}

    def unpopulate(self):
        for binder in self.binders.values():
            binder.unpopulate()

    def populate(self):
        if self.attribute:
            self.collection = getattr(self.object, self.attribute)
        else:
            self.collection = self.object
        self.values = self.ui.values(self.collection)

        self.unpopulate()

        self.binders = {}
        index = 0

        if len(self.values) > len(self.ui.children):
            raise Exception('Number of bind:list children is less than collection size')

        for value in self.values:
            template = self.ui.children[index]
            index += 1
            binder = Binder(value, template)
            binder.autodiscover()
            binder.populate()
            self.binders[value] = binder
            self.ui.post_item_bind(self.object, self.collection, value, template)

        self.ui.post_bind(self.object, self.collection, self.ui)
        return self

    def update(self):
        for value in self.values:
            self.binders[value].update()
            self.ui.post_item_update(self.object, self.collection, value, self.binders[value].ui)


@public
class DictAutoBinding (Binding):
    """
    Binds values from a dict to UI element's children mapping 'bind' attribute to dict key, using :class:`Binder`
    """

    def __init__(self, object, attribute, ui):
        Binding.__init__(self, object, attribute, ui)
        self.binders = {}

    def unpopulate(self):
        for binder in self.binders.values():
            binder.unpopulate()

    def populate(self):
        if self.attribute:
            self.collection = getattr(self.object, self.attribute)
        else:
            self.collection = self.object
        self.values = self.ui.values(self.collection)

        self.unpopulate()

        self.binders = {}
        bindables = self.ui.nearest(
            lambda x: is_bound(x),
            exclude=lambda x: (
                x != self.ui and is_bound_context(x.parent) and x.parent != self.ui
            )
        )

        for bindable in bindables:
            if bindable == self.ui:
                continue
    
            for prop in bindable.properties:            
                if not bindable.properties[prop]:
                    continue
                if prop.startswith('{bind}'):
                    binder = DictValueBinding(self.values, bindable.properties[prop], bindable, prop.split('}')[1])
                elif prop == 'bind':
                    binder = DictValueBinding(self.values, bindable.bind, bindable)
                else:
                    continue
                key = bindable.properties[prop]
                binder.populate()
                self.binders[key] = binder

        self.ui.post_bind(self.object, self.collection, self.ui)
        return self

    def update(self):
        for key in self.binders:
            self.binders[key].update()
            self.ui.post_item_update(self.object, self.collection, key, self.binders[key].ui)


def _element_in_child_binder(root, e):
    # detect if the element is trapped inside a nested bind: tag
    # relative to e
    return any(x.typeid.startswith('bind:') for x in root.path_to(e))


def _element_in_child_template(root, e):
    # detect if the element is trapped inside a nested bind: tag
    # relative to e
    return any(x.typeid.startswith('bind:template') for x in root.path_to(e))


@public
class CollectionAutoBinding (Binding):
    """
    Binds values of a collection to UI element's children using a template.
    The expected UI layout::

        <xml xmlns:bind="bind">
            <bind:collection id="<binding to this>">
                <container-element bind="__items">
                    <1-- instantiated templates will appear here -->
                </container-element>

                <bind:template>
                    <!-- a template for one collection item
                         it will be bound to item using ajenti.ui.binder.Binder -->
                    <label bind="some_property" />

                    <button id="__delete" /> <!-- a delete button may appear in the template -->
                </bind:template>

                <button id="__add" /> <!-- an add button may appear inside collection tag -->
            </bind:collection>
        </xml>
    """

    def __init__(self, object, attribute, ui):
        Binding.__init__(self, object, attribute, ui)
        self.template = ui.find_type('bind:template')
        if self.template:
            self.template_parent = self.template.parent
            self.template.visible = False
        self.items_ui = self.ui.nearest(lambda x: x.bind == '__items')[0] or self.ui
        self.old_items = copy.copy(self.items_ui.children)

        self.item_ui = {}
        self.binders = {}

    def unpopulate(self):
        if self.template:
            self.template_parent.append(self.template)
        self.items_ui.empty()
        # restore original container content
        self.items_ui.children = copy.copy(self.old_items)
        return self

    def get_template(self, item, ui):
        # override for custom item template creation
        return self.template.clone()

    def populate(self):
        if self.template:
            self.template_parent.remove(self.template)

        if self.attribute:
            self.collection = getattr(self.object, self.attribute)
        else:
            self.collection = self.object

        self.values = self.ui.values(self.collection)
        if self.ui.sorting:
            self.values = sorted(self.values, key=self.ui.sorting)

        self.unpopulate()

        # Do it before DOM becomes huge
        try:
            add_button = self.ui.nearest(lambda x: x.bind == '__add')[0]
            if not _element_in_child_binder(self.ui, add_button):
                add_button.on('click', self.on_add)
        except IndexError:
            pass

        if self.ui.pagesize:
            try:
                self.paging = None
                paging = self.ui.nearest(lambda x: x.bind == '__paging')[0]
                if not _element_in_child_binder(self.ui, paging):
                    self.paging = paging
                    paging.on('switch', self.set_page)
                    paging.length = int((len(self.values) - 1) / self.ui.pagesize) + 1
            except IndexError:
                pass

        # remember old template instances and binders
        old_item_ui = self.item_ui
        old_binders = self.binders

        self.item_ui = {}
        self.binders = {}
        for value in self.values:
            # apply the filter property
            if not self.ui.filter(value):
                continue

            if value in old_item_ui:
                # reuse old template
                template = old_item_ui[value]
            else:
                # create new one
                template = self.get_template(value, self.ui)
                template.visible = True
            self.items_ui.append(template)
            self.item_ui[value] = template
            if value in old_binders:
                # reuse binder
                binder = old_binders[value]
            else:
                # create new one
                binder = Binder(value, template)
                binder.autodiscover()
            binder.populate()
            self.binders[value] = binder

            try:
                del_button = template.nearest(lambda x: x.bind == '__delete')[0]
                if not _element_in_child_binder(template, del_button):
                    del_button.on('click', self.on_delete, value)
            except IndexError:
                pass

            self.ui.post_item_bind(self.object, self.collection, value, template)

        self.set_page(0)
        self.ui.post_bind(self.object, self.collection, self.ui)
        return self

    def set_page(self, page=0):
        if self.ui.pagesize:
            i = 0
            for value in self.values:
                self.item_ui[value].visible = int(i / self.ui.pagesize) == page
                i += 1
            if self.paging:
                self.paging.active = page

    def on_add(self):
        self.update()
        self.ui.add_item(self.ui.new_item(self.collection), self.collection)
        self.populate()

    def on_delete(self, item):
        self.update()
        self.ui.delete_item(item, self.collection)
        self.populate()

    def update(self):
        if hasattr(self.items_ui, 'sortable') and self.items_ui.order:
            new_values = []
            for i in self.items_ui.order:
                if i - 1 < len(self.collection):
                    new_values.append(self.collection[i - 1])
            while len(self.collection) > 0:
                self.collection.pop(0)
            for e in new_values:
                self.collection.append(e)
            self.items_ui.order = []
        for value in self.values:
            if self.ui.filter(value):
                self.binders[value].update()
                self.ui.post_item_update(self.object, self.collection, value, self.binders[value].ui)


@public
class Binder (object):
    """
    An automatic object-to-ui-hierarchy binder. Uses ``bind`` UI property to find what and where to bind.

    :param object: Python object
    :param ui: UI hierarchy root
    """

    def __init__(self, object=None, ui=None):
        self.bindings = []
        self.reset(object, ui)

    def reset(self, object=None, ui=None):
        """
        Cancels the binding and replaces Python object / UI root.
        """
        self.unpopulate()
        if object is not None:
            self.object = object
        if ui:
            self.ui = ui
        return self

    def autodiscover(self, object=None, ui=None):
        """
        Recursively scans UI tree for ``bind`` properties, and creates bindings.
        """

        # Initial call
        if not object and not ui:
            self.bindings = []

        ui = ui or self.ui
        object = object or self.object


        bindables = ui.nearest(
            lambda x: is_bound(x),
            exclude=lambda x: (
                x.parent != ui and x != ui and (
                    '{bind}context' in x.parent.properties  # skip nested contexts
                    or x.parent.typeid.startswith('bind:')  # and templates and nested collections
                )
            )
        )

        for bindable in bindables:
            # Custom/collection binding
            if bindable.typeid.startswith('bind:'):
                k = bindable.bind
                if k in dir(object):
                    self.add(bindable.binding(object, k, bindable))
                continue

            for prop in bindable.properties:
                if not prop.startswith('{bind') and prop != 'bind':
                    continue
                k = bindable.properties[prop]

                if prop == 'bind':
                    if k in dir(object):
                        self.add(PropertyBinding(object, k, bindable))

                # Nested binder context
                if prop == '{binder}context':
                    if bindable is not ui and k:
                        self.autodiscover(getattr(object, k), bindable)

                # Property binding
                if prop.startswith('{bind}'):
                    if k in dir(object):
                        self.add(PropertyBinding(object, k, bindable, prop.split('}')[1]))

        return self

    def add(self, binding):
        self.bindings.append(binding)

    def populate(self):
        """
        Populates the bindings.
        """
        for binding in self.bindings:
            binding.populate()
        return self

    def unpopulate(self):
        """
        Unpopulates the bindings.
        """
        for binding in self.bindings:
            binding.unpopulate()
        return self

    def update(self):
        """
        Updates the bindings.
        """
        for binding in self.bindings:
            binding.update()
        return self


# Helper elements
@p('post_bind', default=lambda o, c, u: None, type=eval, public=False,
    doc='Called after binding is complete, ``lambda object, collection, ui: None``')
@p('post_item_bind', default=lambda o, c, i, u: None, type=eval, public=False,
    doc='Called after an item is bound, ``lambda object, collection, item, item-ui: None``')
@p('post_item_update', default=lambda o, c, i, u: None, type=eval, public=False,
    doc='Called after an item is updated, ``lambda object, collection, item, item-ui: None``')
@p('binding', default=ListAutoBinding, type=eval, public=False,
    doc='Collection binding class to use')
@p('filter', default=lambda i: True, type=eval, public=False,
    doc='Called to filter collection''s values, ``lambda value: bool``')
@p('values', default=lambda c: c, type=eval, public=False,
    doc='Called to extract values from the collection, ``lambda collection: []``')
@public
class BasicCollectionElement (UIElement):
    pass


@public
@plugin
class ListElement (BasicCollectionElement):
    typeid = 'bind:list'


@public
@p('add_item', default=lambda i, c: c.append(i), type=eval, public=False,
    doc='Called to append value to the collection, ``lambda item, collection: None``')
@p('new_item', default=lambda c: None, type=eval, public=False,
    doc='Called to create an empty new item, ``lambda collection: object()``')
@p('delete_item', default=lambda i, c: c.remove(i), type=eval, public=False,
    doc='Called to remove value from the collection, ``lambda item, collection: None``')
@p('sorting', default=None, type=eval, public=False,
    doc='If defined, used as key function to sort items')
@p('pagesize', default=0, type=int, public=False)
@p('binding', default=CollectionAutoBinding, type=eval, public=False)
@plugin
class CollectionElement (BasicCollectionElement):
    typeid = 'bind:collection'


@p('binding', default=DictAutoBinding, type=eval, public=False)
@plugin
class DictElement (BasicCollectionElement):
    typeid = 'bind:dict'



from ajenti.api import plugin
from ajenti.plugins.dashboard.api import ConfigurableWidget


@plugin
class MuninWidget (ConfigurableWidget):
    name = 'Munin'
    icon = 'stethoscope'
    hidden = True

    def on_prepare(self):
        self.append(self.ui.inflate('munin:widget'))

    def on_start(self):
        self.find('plot').url = self.config['url']
        if 'period' in self.config:
            self.find('plot').period = self.config['period']

    def create_config(self):
        return {'url': '', 'period': ''}



from passlib.hash import sha512_crypt

from ajenti.api import plugin
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder

from ..mediadb import DB, Client, RootDirectory, ClientDirectoryPermission


@plugin
class MediaLibrarySection (SectionPlugin):
    def init(self):
        self.title = 'Media Library'
        self.icon = 'play-circle'
        self.category = 'Elements'
        self.append(self.ui.inflate('custom_media:admin-main'))

        def post_directory_bind(o, c, i, u):
            u.find('create-permission').on('click', self.on_create_permission, i)

        self.find('clients').delete_item = lambda i,c: self.delete_object(i)
        self.find('directories').delete_item = lambda i,c: self.delete_object(i)
        self.find('permissions').delete_item = lambda i,c: self.delete_object(i)
        self.find('directories').post_item_bind = post_directory_bind
     
        self.clients = []
        self.directories = []
        
        self.binder = Binder(self, self)
        self.binder.autodiscover()
        self.db = DB.instance()

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.clients = self.db.list(Client)
        self.directories = self.db.list(RootDirectory)
        self.binder.unpopulate()

        self.find('user-dropdown').values = [None] + self.clients
        self.find('user-dropdown').labels = [''] + [_.username for _ in self.clients]

        self.binder.populate()

    def delete_object(self, object):
        self.save()
        self.db.delete(object)
        self.refresh()

    @on('create-client', 'click')
    def on_create_client(self):
        self.save()
        c = Client(username='unnamed', password='')
        self.db.create(c)
        self.refresh()

    @on('create-directory', 'click')
    def on_create_directory(self):
        self.save()
        d = RootDirectory(name='unnamed', path='/')
        self.db.create(d)
        self.refresh()

    def on_create_permission(self, directory):
        self.save()
        p = ClientDirectoryPermission(directory=directory, client=None)
        self.db.create(p)
        self.refresh()

    @on('save', 'click')
    def save(self):
        self.binder.update()
        
        for client in self.clients:
            if not client.password.startswith('$'):
                client.password = sha512_crypt.encrypt(client.password)
        self.db.commit()
        self.refresh()



from ajenti.api import *
from ajenti.ui import p, UIElement


@p('text', default='', bindtypes=[str, unicode, int, float])
@plugin
class Label (UIElement):
    typeid = 'label'


@p('text', default='', bindtypes=[str, unicode, int])
@plugin
class Tooltip (UIElement):
    typeid = 'tooltip'


@p('icon', default=None, bindtypes=[str, unicode])
@plugin
class Icon (UIElement):
    typeid = 'icon'


@p('text', default='', bindtypes=[str, unicode])
@p('icon', default=None)
@p('warning', default=None)
@plugin
class Button (UIElement):
    typeid = 'button'


@p('width', default=None)
@p('value', default=0, type=float, bindtypes=[int, float])
@plugin
class ProgressBar (UIElement):
    typeid = 'progressbar'



import os


class Tape (object):
    def __init__(self):
        self.name = '00000'
        self.extras = '0'


class TapeGroup (object):
    def __init__(self):
        self.id = None
        self.name = 'New Group'
        self.tapes = []

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name


class TapeGroupBackend (object):
    def __init__(self):
        self.groups = []
        self.root = '/var/lib/syslink/archive/tapegroups'

    def reload(self):
        self.groups = []
        for f in os.listdir(self.root):
            lines = open(os.path.join(self.root, f)).read().splitlines()
            g = TapeGroup()
            g.id = f
            g.name = lines[0]
            for l in lines[1:]:
                t = Tape()
                t.name, t.extras = l.split(',', 1)
                g.tapes.append(t)
            self.groups.append(g)

        for g in self.groups:
            if g.id == 'blank':
                break
        else:
            g = TapeGroup()
            g.id = 'blank'
            g.name = 'blank'
            self.groups.append(g)
            self.save()

    def save(self):
        for f in os.listdir(self.root):
            os.unlink(os.path.join(self.root, f))

        for g in self.groups:
            if not g.id:
                id = 0
                for f in os.listdir(self.root):
                    if not '.' in f:
                        if f != 'blank' and int(f) >= id:
                            id = int(f) + 1
                g.id = str(id)

            c = ''.join((tape.name + ',' + tape.extras + '\n') for tape in g.tapes)
            c = g.name + '\n' + c
            open(os.path.join(self.root, g.id), 'w').write(c)



import os

from ajenti.api import *
from ajenti.ui import p, UIElement, on


@p('value', default='', bindtypes=[str, unicode, int, long])
@p('readonly', type=bool, default=False)
@p('type', default='text')
@plugin
class TextBox (UIElement):
    typeid = 'textbox'


@p('value', default='', bindtypes=[str, unicode, int, long])
@plugin
class PasswordBox (UIElement):
    typeid = 'passwordbox'


@p('value', default='', bindtypes=[str, unicode])
@p('icon', default=None)
@p('placeholder', default=None)
@plugin
class Editable (UIElement):
    typeid = 'editable'


@p('text', default='')
@p('value', default=False, bindtypes=[bool])
@plugin
class CheckBox (UIElement):
    typeid = 'checkbox'


@p('labels', default=[], type=list)
@p('values', default=[], type=list, public=False)
@p('value', bindtypes=[object])
@p('index', default=0, type=int)
@p('server', default=False, type=bool)
@plugin
class Dropdown (UIElement):
    typeid = 'dropdown'
    
    def value_get(self):
        if self.index < len(self.values):
            try:
                return self.values[self.index]
            except TypeError:
                return None
        return None
        
    def value_set(self, value):
        if value in self.values:
            self.index = self.values.index(value)
        else:
            self.index = 0

    value = property(value_get, value_set)


@p('labels', default=[], type=list)
@p('values', default=[], type=list)
@p('separator', default=None, type=str)
@p('value', default='', bindtypes=[str, unicode])
@plugin
class Combobox (UIElement):
    typeid = 'combobox'


@p('target', type=str)
@plugin
class FileUpload (UIElement):
    typeid = 'fileupload'


@p('active', type=int)
@p('length', type=int)
@plugin
class Paging (UIElement):
    typeid = 'paging'


@p('value', default='', bindtypes=[str, unicode])
@p('directory', default=False, type=bool)
@p('type', default='text')
@plugin
class Pathbox (UIElement):
    typeid = 'pathbox'

    def init(self, *args, **kwargs):
        if self.directory:
            self.dialog = self.ui.create('opendirdialog')
        else:
            self.dialog = self.ui.create('openfiledialog')
        self.append(self.dialog)
        self.dialog.id = 'dialog'
        self.dialog.visible = False

    def on_start(self):
        self.find('dialog').navigate(os.path.split(self.value or '')[0] or '/')
        self.find('dialog').visible = True

    @on('dialog', 'select')
    def on_select(self, path=None):
        self.find('dialog').visible = False
        if path:
            self.value = path



from lxml import etree

import sqlalchemy
from sqlalchemy import Column, Integer, String, Boolean, ForeignKey, BigInteger
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, relation


Base = declarative_base()


class Tape (Base):
    __tablename__ = 'tapes'
    id = Column(Integer, primary_key=True)
    name = Column(String(255))
    uuid = Column(String(255))
    generation = Column(Integer)
    custom_a = Column(String(255))
    custom_b = Column(String(255))
    custom_c = Column(String(255))
    custom_d = Column(String(255))


class Entry (Base):
    __tablename__ = 'entries'
    id = Column(Integer, primary_key=True)
    uid = Column(Integer)
    name = Column(String(255))
    is_dir = Column(Boolean)
    read_only = Column(Boolean)
    length = Column(BigInteger)
    parent_id = Column(Integer, ForeignKey('entries.id', ondelete='CASCADE'), nullable=True)
    parent = relation('Entry', remote_side=[id])
    tape_id = Column(Integer, ForeignKey('tapes.id', ondelete='CASCADE'), nullable=True)
    tape = relation('Tape', uselist=False)

    def path(self):
        e = self
        r = ''
        while e:
            r = '/' + e.name + r
            e = e.parent
        return r


class LTFS (object):
    def __init__(self, url, debug=False):
        self.engine = sqlalchemy.create_engine(url, echo=debug, convert_unicode=True, encoding='utf-8')
        try:
            Base.metadata.create_all(self.engine)
        except:
            pass
        self.session = sessionmaker(bind=self.engine)()

    def delete_tape(self, name=None):
        tapes = []
        if name:
            tapes += self.session.query(Tape).filter(Tape.name == name).all()
        for tape in tapes:
            self.session.delete(tape)
        self.session.commit()

    def get_tape(self, name=None):
        tapes = []
        if name:
            tapes += self.session.query(Tape).filter(Tape.name == name).all()
        return tapes[0] if tapes else None

    def create_tape(self, xml, name, custom_a='', custom_b='', custom_c='', custom_d=''):
        xml = etree.fromstring(xml)

        tape = Tape(
            name=name,
            custom_a=custom_a,
            custom_b=custom_b,
            custom_c=custom_c,
            custom_d=custom_d,
            uuid=xml.find('volumeuuid').text,
            generation=xml.find('generationnumber').text,
        )

        self.session.add(tape)

        def recurse(cwd, element):
            print '%i entries parsed' % recurse.counter
            for eroot in ([element] + [element.find('contents')]):
                if not eroot:
                    continue
                for edir in eroot.getchildren():
                    if edir.tag == 'directory':
                        recurse.counter += 1
                        dir = Entry(
                            parent=cwd,
                            uid=edir.find('fileuid').text,
                            name=edir.find('name').text,
                            read_only=edir.find('readonly').text == 'true',
                            is_dir=True,
                            tape=tape,
                        )
                        self.session.add(dir)
                        recurse(dir, edir)
                for efile in eroot.getchildren():
                    if efile.tag == 'file':
                        recurse.counter += 1
                        file = Entry(
                            parent=cwd,
                            uid=efile.find('fileuid').text,
                            name=efile.find('name').text,
                            read_only=efile.find('readonly').text == 'true',
                            length=int(efile.find('length').text),
                            is_dir=False,
                            tape=tape,
                        )
                        self.session.add(file)

                        if recurse.counter % 100 == 0:
                            self.session.commit()

        recurse.counter = 0
        recurse(None, xml)
        self.session.commit()

    def ls(self, tape, cwd):
        files = self.session.query(Entry).filter(sqlalchemy.and_(Entry.parent == cwd, Entry.tape == tape)).all()
        print len(files), tape, cwd
        
        def _cmp(a, b):
            if a.is_dir == b.is_dir:
                return cmp(a.name, b.name)
            elif b.is_dir:
                return 1
            return -1

        files = sorted(files, cmp=_cmp)
        return files

    def search(self, query, search_files=True, limit=10000, offset=0):
        class SearchResult (object):
            pass

        class FoundNode (object):
            pass

        searchResult = SearchResult()
        searchResult.tapes = []
        searchResult.nodes = []

        found_tapes = self.session.query(Tape) \
            .filter(sqlalchemy.or_(
                Tape.name.ilike('%' + query + '%'),
                Tape.custom_a.ilike('%' + query + '%'),
                Tape.custom_b.ilike('%' + query + '%'),
                Tape.custom_c.ilike('%' + query + '%'),
                Tape.custom_d.ilike('%' + query + '%')
            )) \
            .all()

        searchResult.tapes += list(found_tapes)

        if search_files:
            found_files = self.session.query(Entry).filter(Entry.name.ilike('%' + query + '%')).all()
            searchResult.nodes += list(found_files)

        searchResult.total = len(searchResult.nodes)
        searchResult.nodes = searchResult.nodes[offset:offset+limit]
        return searchResult



import ajenti
from ajenti.api import *
from ajenti.ui import UIElement, p, on
from ajenti.ui.binder import Binder
from ajenti.plugins import BinaryDependency, ModuleDependency


@p('package', default='')
@plugin
class PackageInstaller (UIElement, BasePlugin):
    typeid = 'packages:installer'

    def init(self):
        self.visible = False
        self.append(self.ui.inflate('packages:installer'))
        self.recheck()

    def on_page_load(self):
        self.recheck()

    def recheck(self):
        if ajenti.platform in db:
            apps = db[ajenti.platform]
            if self.package in apps:
                self.pkg = db[ajenti.platform][self.package]
                self.visible = True
                Binder(self, self).autodiscover().populate()
            if self.package.startswith('python-module-'):
                d = ModuleDependency(self.package[len('python-module-')])
            else:
                d = BinaryDependency(self.package)
            if d.satisfied():
                self.visible = False

    @on('install', 'click')
    def on_install(self):
        self.event('activate')
        self.context.launch('install-package', package=self.pkg)


db = {
    'debian': {
        'python-module-BeautifulSoup': 'python-beautifulsoup',
        'supervisord': 'supervisor',
        'hddtemp': 'hddtemp',
        'sensors': 'lm-sensors',
        'munin-cron': 'munin',
        'smbd': 'samba',
        'smartctl': 'smartmontools',
        'squid3': 'squid3',
        'apache2': 'apache2',
        'ctdb': 'ctdb',
        'mysql': 'mysql-server',
        'mysqld_safe': 'mysql-server',
        'psql': 'postgresql',
        'nfsstat': 'nfs-kernel-server',
        'mdadm': 'mdadm',
        'nginx': 'nginx',
        'ipmitool': 'ipmitool',
    },
    'centos': {
        'python-module-BeautifulSoup': 'python-BeautifulSoup',
        'supervisord': 'supervisor',
        'hddtemp': 'hddtemp',
        'sensors': 'lm_sensors',
        'munin-cron': 'munin',
        'smbd': 'samba',
        'smartctl': 'smartmontools',
        'squid3': 'squid',
        'apache2': 'httpd',
        'ctdb': 'ctdb',
        'mysql': 'mysql',
        'mysqld_safe': 'mysql-server',
        'psql': 'postgresql',
        'mdadm': 'mdadm',
        'nginx': 'nginx',
        'ipmitool': 'ipmitool',
    },
    'arch': {
        'python-module-BeautifulSoup': 'python2-beautifulsoup3',
        'supervisord': 'supervisor',
        'hddtemp': 'hddtemp',
        'sensors': 'lm_sensors',
        'munin-cron': 'munin',
        'smbd': 'samba',
        'smartctl': 'smartmontools',
        'squid3': 'squid',
        'apache2': 'apache',
        'mysql': 'mariadb-clients',
        'mysqld_safe': 'mariadb',
        'psql': 'postgresql',
        'mdadm': 'mdadm',
        'nginx': 'nginx',
        'apcaccess': 'apcupsd',
        'openvpn': 'openvpn',
        'nsd': 'nsd',
        'memcached': 'memcached',
        'nfsstat': 'nfs-utils',
        'dhcpd': 'dhcp',
        'named': 'bind',
        'ipmitool': 'ipmitool',
    },
}



import subprocess

from ajenti.api import plugin
from reconfigure.configs import CrontabConfig


@plugin
class CronManager (object):
    def load_tab(self, user):
        self.current_user = user
        try:
            data = subprocess.check_output(['crontab', '-l', '-u', user])
        except Exception:
            data = ''
        config = CrontabConfig(content=data)
        config.load()
        return config

    def save_tab(self, user, config):
        data = config.save()[None]
        p = subprocess.Popen(['crontab', '-', '-u', user])
        stdout, stderr = p.communicate(data + '\n')
        if stderr:
            raise Exception(stderr)
            



from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder

from pyltfs2 import LTFS, Tape, Entry

try:
    import psycogreen.gevent
    psycogreen.gevent.patch_psycopg()
    psycogreen_available = True
except:
    psycogreen_available = False


from ..jobs.backend import LTFSJobsBackend, LTFSJob, LTFSSource
from ..library.backend import LTFSLibraryBackend

CHECKSUM_FILE = '/var/lib/syslink/archive/checksums'


@plugin
class LTFSArchivePlugin (SectionPlugin):
    def init(self):
        self.title = 'Archive'
        self.icon = 'list'
        self.category = 'LTFS'

        self.append(self.ui.inflate('ltfs:archive-main'))
        self.binder = Binder(None, self.find('search-results'))
        self.restore_binder = Binder(None, self.find('restore-dialog'))
        self.tape_binder = Binder(None, self.find('tape-dialog'))
        self.files_binder = Binder(None, self.find('nav'))

        self.editing_tape = None
        self.creating_tape = False
        self.search_offset = 0
        self.last_query = None
        self.display_search(False)

        url = 'mysql+mysqlconnector://root:123@localhost/syslink_tapelibrary'

        self.ltfs = LTFS(url, debug=True)

        def post_tape_bind(object, collection, item, ui):
            lib = LTFSLibraryBackend()
            lib.reload()
            tapenames = [_.barcode for _ in lib.drives]
            tapenames += [_.barcode for _ in lib.slots]
            tapenames += [_.barcode for _ in lib.mailbox]

            def on_browse(tape):
                self.navigate(tape, None)

            def on_edit(tape):
                self.start_editing_tape(tape)

            def on_delete(tape):
                self.context.notify('info', "Tape deleted")
                self.ltfs.delete_tape(name=tape.name)

            ui.find('tape-warning').visible = item.name not in tapenames
            ui.find('edit').on('click', on_edit, item)
            ui.find('delete').on('click', on_delete, item)
            ui.find('browse').on('click', on_browse, item)

        self.find('tapes').post_item_bind = post_tape_bind

        def post_file_bind(object, collection, item, ui):
            def on_click(entry):
                self.navigate(item.tape, item)

            if item.is_dir:
                ui.find('name').on('click', on_click, item)

        self.find('nav_files').post_item_bind = post_file_bind

        def post_node_bind(object, collection, item, ui):
            def on_browse(entry):
                self.navigate(item.tape, item.parent)

            ui.find('browse').on('click', on_browse, item)

        self.find('nodes').post_item_bind = post_node_bind

        self.nav_files = []
        self.results = []
        #self.navigate(self.ltfs.get_tape(name='test'), None)

    def on_page_load(self):
        if not psycogreen_available:
            self.context.notify('err', "psycogreen package unavailable, large queries will time out")

    def navigate(self, tape, cwd):
        self.nav_cwd = cwd
        self.nav_files = self.ltfs.ls(tape, cwd)
        if len(self.nav_files) > 100:
            self.nav_files = self.nav_files[:100]
            #self.context.notify('info', 'Limited to 100 entries...')
        for file in self.nav_files:
            file.selected = False
            file.icon = 'folder-open' if file.is_dir else 'file'
        self.files_binder.reset(self).autodiscover().populate()
        self.find('nav-up').visible = cwd is not None

    @on('nav-up', 'click')
    def on_navigate_up(self):
        self.navigate(self.nav_cwd.tape, self.nav_cwd.parent)

    @on('search-files', 'click')
    def on_search_files(self):
        q = self.find('search-query').value
        if q or len(q) > -2:
            self.search(q)
        else:
            self.context.notify('error', 'Search query is too short!')

    def search(self, query=None, offset=0):
        self.search_offset = offset
        query = query or self.last_query
        self.last_query = query

        self.results = self.ltfs.search(query, limit=100, offset=offset)
        for file in self.results.nodes:
            file.selected = False
        self.binder.reset(self.results).autodiscover().populate()

        self.find('page-indicator').text = 'Page %i (%i results total)' % (offset / 100 + 1, self.results.total)
        self.display_search(True, len(self.results.nodes) > 99 or self.search_offset != 0)

    def display_search(self, visible=True, paging=True):
        self.find('page-indicator').visible = visible
        self.find('page-next').visible = visible and paging
        self.find('page-prev').visible = visible and paging
        self.find('select-all').visible = visible
        self.find('clear').visible = visible

    @on('select-all', 'click')
    def on_select_all(self):
        for file in self.results.nodes:
            file.selected = True
        self.binder.populate()

    @on('page-prev', 'click')
    def on_page_prev(self):
        if self.search_offset > 0:
            self.search(offset=self.search_offset - 100)

    @on('page-next', 'click')
    def on_page_next(self):
        self.search(offset=self.search_offset + 100)

    @on('clear', 'click')
    def on_clear_search(self):
        self.display_search(False)
        self.binder.reset(None)

    @on('search-tapes', 'click')
    def on_search_tapes(self):
        results = self.ltfs.search(self.find('search-query').value, False)
        self.binder.reset(results).autodiscover().populate()

    @on('add', 'click')
    def on_add(self):
        tape = self.ltfs.DBContentTape()
        tape.name = 'New tape'
        tape.root = '/'
        self.creating_tape = True
        self.start_editing_tape(tape)

    @on('restore', 'click')
    def on_restore(self):
        self.binder.update()
        self.files_binder.update()
        j = LTFSJob()
        for file in (self.results.nodes if self.results else []) + self.nav_files:
            if file.selected:
                j.source.append(LTFSSource(file.path()))
                j.selected_tape = file.tape.name
        j.type = 'RESTORE'

        self.creating_job = j
        self.restore_binder.reset(self.creating_job).autodiscover().populate()
        self.find('restore-dialog').visible = True

    @on('restore-dialog', 'button')
    def on_restore_dialog(self, button=None):
        self.restore_binder.update()
        self.find('restore-dialog').visible = False

        if button == 'ok':
            LTFSJobsBackend().create_job(self.creating_job)
            self.context.notify('info', 'Job created')

    @on('restore-destination-select', 'click')
    def on_restore_destination_select(self):
        self.restore_binder.update()
        self.find('restore-destination-dialog').visible = True

    @on('restore-destination-dialog', 'select')
    def on_restore_destination_selected(self, path=None):
        self.find('restore-destination-dialog').visible = False
        self.creating_job.destination = path
        self.restore_binder.reset().autodiscover().populate()

    @on('tape-dialog', 'button')
    def on_dialog(self, button=None):
        if button == 'cancel':
            self.find('tape-dialog').visible = False

        self.tape_binder.update()
        if button == 'save':
            found = self.ltfs.get_tape(name=self.editing_tape.name)
            if found and found != self.editing_tape:
                self.context.notify('error', "This tape already exists. Pick a different name or press Reindex to re-index contents")
            else:
                if self.creating_tape:
                    self.creating_tape = False
                    self.ltfs.lic_CreateTape(
                        self.editing_tape.root,
                        self.editing_tape.name,
                        self.editing_tape.custom_a,
                        self.editing_tape.custom_b,
                        self.editing_tape.custom_c,
                        self.editing_tape.custom_d,
                    )
                    self.context.notify('info', "Indexing complete")
                else:
                    self.ltfs.session.merge(self.editing_tape)
                    self.ltfs.session.commit()
                self.editing_tape = None
                self.find('tape-dialog').visible = False
        if button == 'reindex':
            self.find('tape-dialog').visible = False
            tape = self.ltfs.get_tape(name=self.editing_tape.name)
            self.ltfs.delete_tape(name=tape.name)
            self.ltfs.lic_CreateTape(
                self.editing_tape.root,
                tape.name,
                tape.custom_a,
                tape.custom_b,
                tape.custom_c,
                tape.custom_d,
            )
            self.editing_tape = None
            self.context.notify('info', "Re-indexing complete")

    def start_editing_tape(self, tape):
        self.editing_tape = tape
        if not hasattr(tape, 'root'):
            tape.root = '/'
        self.tape_binder.reset(self.editing_tape).autodiscover().populate()
        self.find('tape-dialog').visible = True



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='Elements Media Library',
    icon='hdd',
    dependencies=[
        PluginDependency('elements'),
    ],
)


def init():
    import clientui.http
    import clientui.main



import logging
import os
import subprocess
import time

from ajenti.api import *

from api import Task, TaskError



@plugin
class CommandTask (Task):
    name = 'Execute command'
    ui = 'tasks:params-execute'
    default_params = {
        'command': '',
    }

    def run(self, command=None):
        p = subprocess.Popen(command, shell=True, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        o,e = p.communicate()
        if p.returncode:
            raise TaskError(o + e)
        else:
            self.result.output = o


@plugin
class CopyFilesTask (Task):
    name = 'Copy files'
    ui = 'tasks:params-copydir'
    command = ['cp', '-rp']
    message_template = _('Copying %s')
    default_params = {
        'source': '',
        'destination': '',
    }

    def run(self, source=None, destination=None):
        index = 0
        files = os.listdir(source)
        if not os.path.exists(destination):
            os.makedirs(destination)
        if not destination.endswith('/'):
            destination += '/'

        for file in files:
            self.message = self.message_template % file
            srcpath = os.path.join(source, file)
            p = subprocess.Popen(self.command + [srcpath, destination])
            while p.poll() is None:
                time.sleep(1)
                if self.aborted:
                    p.terminate()
                    return
            index += 1
            self.set_progress(index, len(files))


@plugin
class MoveFilesTask (CopyFilesTask):
    name = 'Move files'
    command = ['mv']
    message_template = _('Moving %s')



@plugin
class RSyncTask (Task):
    name = 'RSync'
    ui = 'tasks:params-rsync'
    default_params = {
        'source': '',
        'destination': '',
        'archive': True,
        'recursive': True,
        'times': True,
        'xattrs': True,
        'delete': False,
        'perms': True,
    }

    def run(self, source=None, destination=None, **kwargs):
        cmd = ['rsync']
        for opt in ['archive', 'recursive', 'times', 'xattrs', 'delete', 'perms']:
            if kwargs.get(opt, self.default_params[opt]):
                cmd += ['--' + opt]
        cmd += [source, destination]
        logging.info('RSync: ' + ' '.join(cmd))
        p = subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE)  
        o, e = p.communicate()
        if p.returncode:
            raise TaskError(o + e)



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='Munin',
    icon='stethoscope',
    dependencies=[
        PluginDependency('main'),
        PluginDependency('dashboard'),
        BinaryDependency('munin-cron'),
        ModuleDependency('BeautifulSoup'),
    ],
)


def init():
    import main
    import widget



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='APC UPS Status',
    icon='bolt',
    dependencies=[
        PluginDependency('main'),
        PluginDependency('dashboard'),
        BinaryDependency('apcaccess'),
    ],
)


def init():
    import widget



import subprocess

from ajenti.api import *
from ajenti.util import cache_value

from api import Service, ServiceManager


@plugin
class FreeBSDServiceManager (ServiceManager):
    platforms = ['freebsd']

    @cache_value(1)
    def get_all(self):
        r = []
        for line in subprocess.check_output(['service', '-l']).splitlines():
            if line:
                s = FreeBSDService(line)
                try:
                    s.refresh()
                    r.append(s)
                except OSError:
                    pass
        return r


class FreeBSDService (Service):
    source = 'rc.d'

    def __init__(self, name):
        self.name = name
        self.script = '/etc/rc.d/%s' % self.name

    def refresh(self):
        self.running = subprocess.call([self.script, 'status']) == 0

    def start(self):
        self.command('start')

    def stop(self):
        self.command('stop')

    def restart(self):
        self.command('restart')

    def command(self, cmd):
        subprocess.call([self.script, cmd])
